package com.sun.imageio.plugins.jpeg;

import javax.imageio.ImageTypeSpecifier;
import javax.imageio.plugins.jpeg.JPEGHuffmanTable;
import javax.imageio.plugins.jpeg.JPEGQTable;
import java.util.ArrayList;
import javax.imageio.metadata.IIOMetadataFormatImpl;

abstract class JPEGMetadataFormat extends IIOMetadataFormatImpl
{
    private static final int MAX_JPEG_DATA_SIZE = 65533;
    String resourceBaseName;
    
    JPEGMetadataFormat(String rootName, int childPolicy) {
        super(rootName, childPolicy);
        this.setResourceBaseName(this.resourceBaseName = this.getClass().getName() + "Resources");
    }
    
    void addStreamElements(String parentName) {
        this.addElement("dqt", parentName, 1, 4);
        this.addElement("dqtable", "dqt", 0);
        this.addAttribute("dqtable", "elementPrecision", 2, false, "0");
        ArrayList<String> list = new ArrayList();
        list.add("0");
        list.add("1");
        list.add("2");
        list.add("3");
        this.addAttribute("dqtable", "qtableId", 2, true, null, list);
        this.addObjectValue("dqtable", JPEGQTable.class, true, null);
        this.addElement("dht", parentName, 1, 4);
        this.addElement("dhtable", "dht", 0);
        ArrayList<String> enumeratedValues = new ArrayList();
        enumeratedValues.add("0");
        enumeratedValues.add("1");
        this.addAttribute("dhtable", "class", 2, true, null, enumeratedValues);
        this.addAttribute("dhtable", "htableId", 2, true, null, list);
        this.addObjectValue("dhtable", JPEGHuffmanTable.class, true, null);
        this.addElement("dri", parentName, 0);
        this.addAttribute("dri", "interval", 2, true, null, "0", "65535", true, true);
        this.addElement("com", parentName, 0);
        this.addAttribute("com", "comment", 0, false, null);
        this.addObjectValue("com", byte[].class, 1, 65533);
        this.addElement("unknown", parentName, 0);
        this.addAttribute("unknown", "MarkerTag", 2, true, null, "0", "255", true, true);
        this.addObjectValue("unknown", byte[].class, 1, 65533);
    }
    
    @Override
    public boolean canNodeAppear(String s, ImageTypeSpecifier imageTypeSpecifier) {
        return this.isInSubtree(s, this.getRootName());
    }
    
    protected boolean isInSubtree(String elementName, String anObject) {
        if (elementName.equals(anObject)) {
            return true;
        }
        String[] childNames = this.getChildNames(elementName);
        int i = 0;
        while (i < childNames.length) {
            if (this.isInSubtree(elementName, childNames[i])) {
                return true;
            }
            ++i;
        }
        return false;
    }
}
